@@ -153,6 +153,7 @@ namespace stan {
         sigs_ = new function_signatures;
       return *sigs_;
     }
+
     void
     function_signatures::set_user_defined(const
                                           std::pair<std::string,
@@ -160,12 +161,14 @@ namespace stan {
                                           name_sig) {
       user_defined_set_.insert(name_sig);
     }
+
     bool
     function_signatures::is_user_defined(const std::pair<std::string,
                                                          function_signature_t>&
                                               name_sig) {
       return user_defined_set_.find(name_sig) != user_defined_set_.end();
     }
+
     bool function_signatures::is_defined(const std::string& name,
                                          const function_signature_t& sig) {
       if (sigs_map_.find(name) == sigs_map_.end())
@@ -177,6 +180,24 @@ namespace stan {
       return false;
     }
 
+    bool function_signatures::discrete_first_arg(const std::string& fun)
+      const {
+      using std::map;
+      using std::string;
+      using std::vector;
+      map<string, vector<function_signature_t> >::const_iterator it
+        = sigs_map_.find(fun);
+      if (it == sigs_map_.end())
+        return false;
+      const vector<function_signature_t> sigs = it->second;
+      for (size_t i = 0; i < sigs.size(); ++i) {
+        if (sigs[i].second.size() == 0
+            || sigs[i].second[0].base_type_ != INT_T)
+          return false;
+      }
+      return true;
+    }
+
     void function_signatures::add(const std::string& name,
                                    const expr_type& result_type,
                                    const std::vector<expr_type>& arg_types) {
@@ -284,6 +305,16 @@ namespace stan {
     void function_signatures::add_unary(const::std::string& name) {
       add(name, DOUBLE_T, DOUBLE_T);
     }
+    void function_signatures::add_unary_vectorized(const::std::string&
+    name) {
+      for (size_t i = 0; i < 8; ++i) {
+        add(name, expr_type(DOUBLE_T, i), expr_type(INT_T, i));
+        add(name, expr_type(DOUBLE_T, i), expr_type(DOUBLE_T, i));
+        add(name, expr_type(VECTOR_T, i), expr_type(VECTOR_T, i));
+        add(name, expr_type(ROW_VECTOR_T, i), expr_type(ROW_VECTOR_T, i));
+        add(name, expr_type(MATRIX_T, i), expr_type(MATRIX_T, i));
+      }
+    }
     void function_signatures::add_binary(const::std::string& name) {
       add(name, DOUBLE_T, DOUBLE_T, DOUBLE_T);
     }
@@ -1246,6 +1277,15 @@ namespace stan {
       return vo == data_origin || vo == transformed_data_origin;
     }
 
+    bool is_fun_origin(const var_origin& vo) {
+      return vo == function_argument_origin
+        || vo == function_argument_origin_lp
+        || vo == function_argument_origin_rng
+        || vo == void_function_argument_origin
+        || vo == void_function_argument_origin_lp
+        || vo == void_function_argument_origin_rng;
+    }
+
     void print_var_origin(std::ostream& o, const var_origin& vo) {
       if (vo == model_name_origin)
         o << "model name";
@@ -1286,31 +1326,42 @@ namespace stan {
                                  const std::vector<expression>& dims,
                                  const base_expr_type& base_type)
       : name_(name), dims_(dims), base_type_(base_type) {  }
+    base_var_decl::base_var_decl(const std::string& name,
+                                 const std::vector<expression>& dims,
+                                 const base_expr_type& base_type,
+                                 const expression& def)
+      : name_(name), dims_(dims), base_type_(base_type), def_(def) {  }
 
     bool variable_map::exists(const std::string& name) const {
       return map_.find(name) != map_.end();
     }
+
     base_var_decl variable_map::get(const std::string& name) const {
       if (!exists(name))
         throw std::invalid_argument("variable does not exist");
       return map_.find(name)->second.first;
     }
+
     base_expr_type variable_map::get_base_type(const std::string& name) const {
       return get(name).base_type_;
     }
+
     size_t variable_map::get_num_dims(const std::string& name) const {
       return get(name).dims_.size();
     }
+
     var_origin variable_map::get_origin(const std::string& name) const {
       if (!exists(name))
         throw std::invalid_argument("variable does not exist");
       return map_.find(name)->second.second;
     }
+
     void variable_map::add(const std::string& name,
                            const base_var_decl& base_decl,
                            const var_origin& vo) {
       map_[name] = range_t(base_decl, vo);
     }
+
     void variable_map::remove(const std::string& name) {
       map_.erase(name);
     }
@@ -1318,24 +1369,23 @@ namespace stan {
     int_var_decl::int_var_decl()
       : base_var_decl(INT_T)
     { }
-
     int_var_decl::int_var_decl(range const& range,
                                std::string const& name,
-                               std::vector<expression> const& dims)
-      : base_var_decl(name, dims, INT_T),
+                               std::vector<expression> const& dims,
+                               expression const& def)
+      : base_var_decl(name, dims, INT_T, def),
         range_(range)
     { }
 
 
-
     double_var_decl::double_var_decl()
       : base_var_decl(DOUBLE_T)
     { }
-
     double_var_decl::double_var_decl(range const& range,
                                      std::string const& name,
-                                     std::vector<expression> const& dims)
-      : base_var_decl(name, dims, DOUBLE_T),
+                                     std::vector<expression> const& dims,
+                                     expression const& def)
+      : base_var_decl(name, dims, DOUBLE_T, def),
         range_(range)
     { }
 
@@ -1379,44 +1429,46 @@ namespace stan {
     positive_ordered_var_decl::positive_ordered_var_decl(expression const& K,
                            std::string const& name,
                            std::vector<expression> const& dims)
-        : base_var_decl(name, dims, VECTOR_T),
-          K_(K) {
-      }
+      : base_var_decl(name, dims, VECTOR_T),
+        K_(K) {
+    }
 
     vector_var_decl::vector_var_decl() : base_var_decl(VECTOR_T) { }
 
     vector_var_decl::vector_var_decl(range const& range,
                                      expression const& M,
                                      std::string const& name,
-                                     std::vector<expression> const& dims)
-        : base_var_decl(name, dims, VECTOR_T),
-          range_(range),
-          M_(M) {
+                                     std::vector<expression> const& dims,
+                                     expression const& def)
+      : base_var_decl(name, dims, VECTOR_T, def),
+        range_(range),
+        M_(M) {
     }
 
     row_vector_var_decl::row_vector_var_decl() : base_var_decl(ROW_VECTOR_T) { }
     row_vector_var_decl::row_vector_var_decl(range const& range,
-                                        expression const& N,
-                                        std::string const& name,
-                                        std::vector<expression> const& dims)
-        : base_var_decl(name, dims, ROW_VECTOR_T),
-          range_(range),
-          N_(N) {
+                                           expression const& N,
+                                           std::string const& name,
+                                           std::vector<expression> const& dims,
+                                           expression const& def)
+      : base_var_decl(name, dims, ROW_VECTOR_T, def),
+        range_(range),
+        N_(N) {
     }
 
     matrix_var_decl::matrix_var_decl() : base_var_decl(MATRIX_T) { }
     matrix_var_decl::matrix_var_decl(range const& range,
-                      expression const& M,
-                      expression const& N,
-                      std::string const& name,
-                      std::vector<expression> const& dims)
-        : base_var_decl(name, dims, MATRIX_T),
-          range_(range),
-          M_(M),
-          N_(N) {
+                                     expression const& M,
+                                     expression const& N,
+                                     std::string const& name,
+                                     std::vector<expression> const& dims,
+                                     expression const& def)
+      : base_var_decl(name, dims, MATRIX_T, def),
+        range_(range),
+        M_(M),
+        N_(N) {
     }
 
-
     cholesky_factor_var_decl::cholesky_factor_var_decl()
       : base_var_decl(MATRIX_T) {
     }
@@ -1457,8 +1509,6 @@ namespace stan {
     }
 
 
-
-
     name_vis::name_vis() { }
     std::string name_vis::operator()(const nil& /* x */) const {
       return "";  // fail if arises
@@ -1561,6 +1611,183 @@ namespace stan {
       return x.base_type_;
     }
 
+    var_decl_dims_vis::var_decl_dims_vis() { }
+    std::vector<expression> var_decl_dims_vis::operator()(const nil& /* x */)
+      const {
+      return std::vector<expression>();  // should not be called
+    }
+    std::vector<expression> var_decl_dims_vis::operator()(const int_var_decl& x)
+      const {
+      return x.dims_;
+    }
+    std::vector<expression> var_decl_dims_vis::operator()(
+                                          const double_var_decl& x)
+      const {
+      return x.dims_;
+    }
+    std::vector<expression> var_decl_dims_vis::operator()(
+                                          const vector_var_decl& x)
+      const {
+      return x.dims_;
+    }
+    std::vector<expression> var_decl_dims_vis::operator()(
+                                          const row_vector_var_decl& x) const {
+      return x.dims_;
+    }
+    std::vector<expression> var_decl_dims_vis::operator()(
+                                          const matrix_var_decl& x)
+      const {
+      return x.dims_;
+    }
+    std::vector<expression> var_decl_dims_vis::operator()(
+                                          const unit_vector_var_decl& x) const {
+      return x.dims_;
+    }
+    std::vector<expression> var_decl_dims_vis::operator()(
+                                          const simplex_var_decl& x) const {
+      return x.dims_;
+    }
+    std::vector<expression> var_decl_dims_vis::operator()(
+                                          const ordered_var_decl& x) const {
+      return x.dims_;
+    }
+    std::vector<expression> var_decl_dims_vis::operator()(
+                                     const positive_ordered_var_decl& x) const {
+      return x.dims_;
+    }
+    std::vector<expression> var_decl_dims_vis::operator()(
+                                     const cholesky_factor_var_decl& x) const {
+      return x.dims_;
+    }
+    std::vector<expression> var_decl_dims_vis::operator()(
+                                     const cholesky_corr_var_decl& x) const {
+      return x.dims_;
+    }
+    std::vector<expression> var_decl_dims_vis::operator()(
+                                     const cov_matrix_var_decl& x) const {
+      return x.dims_;
+    }
+    std::vector<expression> var_decl_dims_vis::operator()(
+                                     const corr_matrix_var_decl& x) const {
+      return x.dims_;
+    }
+
+    var_decl_has_def_vis::var_decl_has_def_vis() { }
+    bool var_decl_has_def_vis::operator()(const nil& /* x */)
+      const {
+      return false;  // should not be called
+    }
+    bool var_decl_has_def_vis::operator()(const int_var_decl& x)
+      const {
+      return !is_nil(x.def_);
+    }
+    bool var_decl_has_def_vis::operator()(const double_var_decl& x)
+      const {
+      return !is_nil(x.def_);
+    }
+    bool var_decl_has_def_vis::operator()(const vector_var_decl& x)
+      const {
+      return !is_nil(x.def_);
+    }
+    bool var_decl_has_def_vis::operator()(
+                                    const row_vector_var_decl& x) const {
+      return !is_nil(x.def_);
+    }
+    bool var_decl_has_def_vis::operator()(const matrix_var_decl& x)
+      const {
+      return !is_nil(x.def_);
+    }
+    bool var_decl_has_def_vis::operator()(
+                                    const unit_vector_var_decl& x) const {
+      return !is_nil(x.def_);
+    }
+    bool var_decl_has_def_vis::operator()(
+                                     const simplex_var_decl& x) const {
+      return !is_nil(x.def_);
+    }
+    bool var_decl_has_def_vis::operator()(
+                                     const ordered_var_decl& x) const {
+      return !is_nil(x.def_);
+    }
+    bool var_decl_has_def_vis::operator()(
+                             const positive_ordered_var_decl& x) const {
+      return !is_nil(x.def_);
+    }
+    bool var_decl_has_def_vis::operator()(
+                                     const cholesky_factor_var_decl& x) const {
+      return !is_nil(x.def_);
+    }
+    bool var_decl_has_def_vis::operator()(
+                                     const cholesky_corr_var_decl& x) const {
+      return !is_nil(x.def_);
+    }
+    bool var_decl_has_def_vis::operator()(
+                                     const cov_matrix_var_decl& x) const {
+      return !is_nil(x.def_);
+    }
+    bool var_decl_has_def_vis::operator()(
+                                     const corr_matrix_var_decl& x) const {
+      return !is_nil(x.def_);
+    }
+
+
+    var_decl_def_vis::var_decl_def_vis() { }
+    expression var_decl_def_vis::operator()(const nil& /* x */)
+      const {
+      return expression();  // should not be called
+    }
+    expression var_decl_def_vis::operator()(const int_var_decl& x)
+      const {
+      return x.def_;
+    }
+    expression var_decl_def_vis::operator()(const double_var_decl& x)
+      const {
+      return x.def_;
+    }
+    expression var_decl_def_vis::operator()(const vector_var_decl& x)
+      const {
+      return x.def_;
+    }
+    expression var_decl_def_vis::operator()(
+                                   const row_vector_var_decl& x) const {
+      return x.def_;
+    }
+    expression var_decl_def_vis::operator()(const matrix_var_decl& x)
+      const {
+      return x.def_;
+    }
+    expression var_decl_def_vis::operator()(
+                                   const unit_vector_var_decl& x) const {
+     return x.def_;
+    }
+    expression var_decl_def_vis::operator()(
+                                   const simplex_var_decl& x) const {
+      return x.def_;
+    }
+    expression var_decl_def_vis::operator()(
+                                   const ordered_var_decl& x) const {
+      return x.def_;
+    }
+    expression var_decl_def_vis::operator()(
+                                   const positive_ordered_var_decl& x) const {
+      return x.def_;
+    }
+    expression var_decl_def_vis::operator()(
+                                   const cholesky_factor_var_decl& x) const {
+      return x.def_;
+    }
+    expression var_decl_def_vis::operator()(
+                                   const cholesky_corr_var_decl& x) const {
+      return x.def_;
+    }
+    expression var_decl_def_vis::operator()(
+                                   const cov_matrix_var_decl& x) const {
+      return x.def_;
+    }
+    expression var_decl_def_vis::operator()(
+                                   const corr_matrix_var_decl& x) const {
+      return x.def_;
+    }
 
 
    // can't template out in .cpp file
@@ -1590,6 +1817,18 @@ namespace stan {
       return boost::apply_visitor(var_decl_base_type_vis(), decl_);
     }
 
+    std::vector<expression> var_decl::dims() const {
+      return boost::apply_visitor(var_decl_dims_vis(), decl_);
+    }
+
+    bool var_decl::has_def() const {
+      return boost::apply_visitor(var_decl_has_def_vis(), decl_);
+    }
+
+    expression var_decl::def() const {
+      return boost::apply_visitor(var_decl_def_vis(), decl_);
+    }
+
     statement::statement() : statement_(nil()) { }
 
     statement::statement(const statement_t& st) : statement_(st) { }
@@ -1757,6 +1996,9 @@ namespace stan {
                && expr_.expression_type()
                   != truncation_.high_.expression_type());
     }
+    bool sample::is_discrete() const {
+      return is_discrete_;
+    }
 
     assignment::assignment() {
     }
