@@ -20,6 +20,7 @@ namespace stan {
      * @param stan_lang_in Stan model specification
      * @param cpp_out C++ code output stream
      * @param model_name Name of model class
+     * @param allow_undefined Permit undefined functions?
      *
      * @return <code>false</code> if code could not be generated
      *    due to syntax error in the Stan model;
@@ -28,10 +29,11 @@ namespace stan {
     bool compile(std::ostream* msgs,
                  std::istream& stan_lang_in,
                  std::ostream& cpp_out,
-                 const std::string& model_name) {
+                 const std::string& model_name,
+                 const bool allow_undefined = false) {
       program prog;
       bool parsed_ok = parse(msgs, stan_lang_in,
-                             model_name, prog);
+                             model_name, prog, allow_undefined);
       if (!parsed_ok)
         return false;  // syntax error in program
       generate_cpp(prog, model_name, cpp_out);
