@@ -45,7 +45,7 @@ namespace stan {
         >> lit("?")
         >> expression_r(_r1)
         >> lit(":")
-        >> expression_r(_r1)[validate_conditional_op_f(_val, _pass,
+        >> expression_r(_r1)[validate_conditional_op_f(_val, _r1, _pass,
                                      boost::phoenix::ref(var_map_),
                                      boost::phoenix::ref(error_msgs))];
 
