@@ -1,29 +1,34 @@
 #ifndef STAN_MCMC_HMC_NUTS_ADAPT_DENSE_E_NUTS_HPP
 #define STAN_MCMC_HMC_NUTS_ADAPT_DENSE_E_NUTS_HPP
 
+#include <stan/interface_callbacks/writer/base_writer.hpp>
 #include <stan/mcmc/stepsize_covar_adapter.hpp>
 #include <stan/mcmc/hmc/nuts/dense_e_nuts.hpp>
 
 namespace stan {
-
   namespace mcmc {
-
-    // The No-U-Turn Sampler (NUTS) on a
-    // Euclidean manifold with dense metric
-    // and adaptive stepsize
-    template <typename Model, class BaseRNG>
+    /**
+     * The No-U-Turn sampler (NUTS) with multinomial sampling
+     * with a Gaussian-Euclidean disintegration and adaptive
+     * dense metric and adaptive step size
+     */
+    template <class Model, class BaseRNG>
     class adapt_dense_e_nuts : public dense_e_nuts<Model, BaseRNG>,
                                public stepsize_covar_adapter {
     public:
-        adapt_dense_e_nuts(Model &model, BaseRNG& rng,
-                           std::ostream* o, std::ostream* e)
-          : dense_e_nuts<Model, BaseRNG>(model, rng, o, e),
+        adapt_dense_e_nuts(const Model& model, BaseRNG& rng)
+          : dense_e_nuts<Model, BaseRNG>(model, rng),
           stepsize_covar_adapter(model.num_params_r()) {}
 
       ~adapt_dense_e_nuts() {}
 
-      sample transition(sample& init_sample) {
-        sample s = dense_e_nuts<Model, BaseRNG>::transition(init_sample);
+      sample
+      transition(sample& init_sample,
+                 interface_callbacks::writer::base_writer& info_writer,
+                 interface_callbacks::writer::base_writer& error_writer) {
+        sample s = dense_e_nuts<Model, BaseRNG>::transition(init_sample,
+                                                            info_writer,
+                                                            error_writer);
 
         if (this->adapt_flag_) {
           this->stepsize_adaptation_.learn_stepsize(this->nom_epsilon_,
@@ -33,7 +38,7 @@ namespace stan {
                                                                  this->z_.q);
 
           if (update) {
-            this->init_stepsize();
+            this->init_stepsize(info_writer, error_writer);
 
             this->stepsize_adaptation_.set_mu(log(10 * this->nom_epsilon_));
             this->stepsize_adaptation_.restart();
@@ -49,7 +54,5 @@ namespace stan {
     };
 
   }  // mcmc
-
 }  // stan
-
 #endif
