@@ -5,6 +5,7 @@
 
 #include <stan/services/arguments/arg_static.hpp>
 #include <stan/services/arguments/arg_nuts.hpp>
+#include <stan/services/arguments/arg_xhmc.hpp>
 
 namespace stan {
   namespace services {
@@ -17,6 +18,7 @@ namespace stan {
 
         _values.push_back(new arg_static());
         _values.push_back(new arg_nuts());
+        // _values.push_back(new arg_xhmc());
 
         _default_cursor = 1;
         _cursor = _default_cursor;
@@ -27,4 +29,3 @@ namespace stan {
 }  // stan
 
 #endif
-
